<?php
/**
 * The Auth_cas class provides an CAS implementation of the Horde
 * authentication system.
 *
 * $Horde: framework/Auth/Auth/cas.php,v 1.2 2009/02/23 08:28:30 $
 *
 * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (LGPL). If you did
 * not receive this file, see http://opensource.org/licenses/lgpl-license.php.
 *
 * @author  Julien Marchal <julien.marchal@univ-nancy2.fr>
 * @package Horde_Auth
 */

require_once 'CAS/CAS.php';
require_once HORDE_BASE . '/lib/Horde/Notification.php';

define('AUTH_REASON_BADPERM', 'noperm');

class Auth_cas extends Auth 
{
	var $_cascli=null;
	var $_params;
	var $_logout;
	var $_maxloop = 3;
	var $_timeloop = 10;
	var $_paramStopAuth = "stopAuth";
	
	var $_imapService = "";
	var $_imapRealm = "";
	var $_imapHordeAuth = false;
	
	var $capabilities = array('add'           => false,
		'update'        => false,
		'resetpassword' => false,
		'remove'        => false,
		'list'          => false,
		'groups'        => false,
		'transparent'   => true);
	
	
	function Auth_cas($params = array()) 
		{        
		if (!Util::extensionExists('curl')) {
			Horde::fatal(_("Auth_cas: Required CURL extension not found."), __FILE__, __LINE__);
		}
		if (!Util::extensionExists('openssl')) {
			Horde::fatal(_("Auth_cas: Required OpenSSL extension not found."), __FILE__, __LINE__);
		}
		if (!Util::extensionExists('domxml')) {
			if (!version_compare(PHP_VERSION,'5','>='))
    		{
				Horde::fatal(_("Auth_cas: Required DOM extension not found."), __FILE__, __LINE__);
    		}
		}
		if (!Util::extensionExists('zlib')) {
			Horde::fatal(_("Auth_cas: Required Zlib extension not found."), __FILE__, __LINE__);
		}
		
		require_once 'CAS/CAS.php';
		
		$this->_setParams($params);
		
		if($this->_params['debug']) {
			phpCAS::setDebug($this->_params['debug_file']);
		}
		
		if(empty($this->_params['hostpath'])) {
			$this->_params['hostpath']='';
		}
		
		$this->__get_arg(); 
		
		phpCAS::proxy(CAS_VERSION_2_0,
			$this->_params['hostspec'],
			$this->_params['hostport'],
			$this->_params['hostpath']);
		
		phpCAS::setPGTStorageFile('plain',$this->_params['tmpdir']);
		phpCAS::setFixedCallbackURL($this->_params['proxyback']);
		phpCAS::setFixedServiceURL($this->__getServiceURL());
		phpCAS::setNoCasServerValidation();
		
		if(isset($_GET['pgtIou'])) {
			if(phpCAS::isAuthenticated()) {
				phpCAS::forceAuthentication();
			}
		}
		
		phpCAS::handleLogoutRequests($this->_params['handleLogout'], $this->_params['handleLogoutHost']);
		
		$this->_logout = $this->getLogoutReason();
		
		// ajouter test failed pour imp
		if ($this->_logout == AUTH_REASON_FAILED) {
			$this->__refreshAuth();
		}	      
		}
	
	/**
	 * Finds out if a set of login credentials are valid, and if requested,
	 * mark the user as logged in in the current session.
	 *
	 * @access public
	 *
	 * @param string $userId      The userId to check.
	 * @param array $credentials  The credentials to check.
	 * @param boolean $login      Whether to log the user in. If false, we'll
	 *                            only test the credentials and won't modify
	 *                            the current session. Defaults to true.
	 * @param string $realm       The authentication realm to check.
	 *
	 * @return boolean  Whether or not the credentials are valid.
	 */
	function authenticate($userId, $credentials, $login = true, $realm = null)
		{
		$auth = parent::authenticate($userId, $credentials, $login, $realm);
		return($auth);
		}
	
	/**
	 * Automatic authentication: Find out if the client has HTTP
	 * authentication info present.
	 *
	 * ESUP Comm : Appele une seul fois lors de l'authentification
	 *
	 * @access public
	 *
	 * @return boolean  Whether or not the client is allowed.
	 */
	function transparent() 
		{
		if(phpCAS::isAuthenticated()) {
			$username = phpCAS::getUser();
			
			if($this->_params['authorisation']) {
				$valide = Horde::callHook('_cas_hook_authorisation', $username);
				if( !$valide) {
					Auth::clearAuth();
					@session_destroy();
					
					$login = Horde::url($GLOBALS['registry']->get('webroot', $app) . '/login.php', true);
					if (!empty($url)) {
						$login = Util::addParameter($login, 'url', $url);
					}
					$login = Util::addParameter($login, AUTH_REASON_PARAM, AUTH_REASON_BADPERM, false);
					$login = Util::addParameter($login, $this->_paramStopAuth, "true", false);
					header('Location: ' . $login);
					exit;  
				}
			}
			
			$this->__getIMPVars();
			
			$pt = $this->__getPT($this->_imapService);
			if(!empty($this->_imapRealm)) {
				$uniqueid = $username."@".$this->_imapRealm;
			} else {
				$uniqueid = $username;
			}
			$this->setAuth($uniqueid,
				array('password' => $pt));
			
			
			if( $this->_imapHordeAuth && !empty($this->_imapService)) {
				$GLOBALS['registry']->call('mail/authenticate',array($username,array('password' => $pt),array('realm' => $this->_imapRealm)));
			}	  
			return true;  
		} 
		return false;	  	  
		}
	
	/**
	 * Set configuration parameters
	 *
	 * @access private
	 *
	 * @param array $params  A hash containing connection parameters.
	 */
	function _setParams($params)
		{
		$this->_params = $params;
		}
	
	/**
	 * Return the URI of the login screen for this authentication
	 * object.
	 *
	 * @access private
	 *
	 * @param optional string $app  The application to use.
	 * @param optional string $url  The URL to redirect to after login.
	 *
	 * @return string  The login screen URI.
	 */
	function _getLoginScreen($app = 'horde', $url = '')
		{	  
		$this->__notLoop();
		if(!empty($this->_logout)) {	
			Auth::clearAuth();
			@session_destroy();
			if($this->_params['logout'] == false) {	      
				$login = Horde::url($GLOBALS['registry']->get('webroot', $app) . '/login.php', true);
				if (!empty($url)) {
					$login = Util::addParameter($login, 'url', $url);
				}
				return $login;	
			} else {
				if($_GET[$this->_paramStopAuth]==true){
					$login = Horde::url($GLOBALS['registry']->get('webroot', $app) . '/login.php', true);
					if (!empty($url)) {
						$login = Util::addParameter($login, 'url', $url);
					}
					return $login;
				} else {
					return (phpCAS::getServerLogoutURL().'?service='.preg_replace('/&/','%26',phpCAS::getServiceURL()));
				}
			}
		} else{
			$this->__sav_arg();
			return phpCAS::getServerLoginURL();
		}
		}
	
	function __notLoop($clean = false)
		{
		if($clean) {
			$_SESSION['cas']['countLoop'] = 0;
			unset($_SESSION['cas']['countLoop']);
			
			return(true);
		}
		
		if(empty($_SESSION['cas']['fatalLoop'])) {
			$_SESSION['cas']['fatalLoop']=false;
		}
		
		if(empty($_SESSION['cas']['timeLoop'])) {
			$_SESSION['cas']['timeLoop']=time();
		}
		
		if(!empty($_SESSION['cas']['timeLoop'])) {
			$current=time();
			$calc=$_SESSION['cas']['timeLoop'] + $this->_timeloop;      
			
			if($current >= $calc){
				$_SESSION['cas']['countLoop'] = 0;
				unset($_SESSION['cas']['timeLoop']);
			}
		}
		
		if(empty($_SESSION['cas']['countLoop'])) {
			$_SESSION['cas']['countLoop'] = 0;
		}
		$_SESSION['cas']['countLoop']++ ;
		
		if( ($_SESSION['cas']['countLoop'] > $this->_maxloop) || $_SESSION['cas']['fatalLoop']) {
			$_SESSION['cas']['fatalLoop'] = true;
			@session_destroy();
			$retry_url = phpCAS::getServerLoginURL();
			
			$msg = sprintf(_("<a href='%s'>An error occurred. You can retry connection here</a>"),$retry_url); 
			
			Horde::fatal($msg , __FILE__, __LINE__, false);	 
		}
		}    
	
	/**
	 * Get a new Proxy Ticket
	 *
	 * ESUP Comm : Appele a chaque erreur d'authentification imap ou sieve
	 *
	 * @access public
	 *
	 * @return boolean  Whether or not the client is allowed.
	 */
	function getNewPT($service=''){      
		if(phpCAS::isAuthenticated()) {
			$username = phpCAS::getUser();
			$this->__getIMPVars();
			if ($service!='') 
				$this->_imapService=$service; 
			
			$pt = $this->__getPT($this->_imapService,true);
			
			Auth::clearAuth();
			$_SESSION['imp']['pass'] = Secret::write(Secret::getKey('imp'), $pt);
			
			if(!empty($this->_imapRealm)) {
				$uniqueid = $username."@".$this->_imapRealm;
			} else {
				$uniqueid = $username;
			}
			
			$this->setAuth($uniqueid,
				array('password' => $pt));
			return($pt);
		} else {
			$this->__refreshAuth();
		}
	}
	
	function __getServiceURL(){
		$url = ($_SERVER['HTTPS'] == 'on') ? 'https' : 'http';
		$url .= '://';
		
		// patch DOMINIQUE LALOT (Load balancing)
		//$url .= $_SERVER['SERVER_NAME'];
		$url .= $GLOBALS['conf']['server']['name']; 
		
		if ( ($_SERVER['HTTPS']=='on' && $_SERVER['SERVER_PORT']!=443)
				|| ($_SERVER['HTTPS']!='on' && $_SERVER['SERVER_PORT']!=80) ) {
			$url .= ':';
			$url .= $_SERVER['SERVER_PORT'];
		}      
		
		$url .= strtok($_SERVER['REQUEST_URI'],"?");
		return $url;
	}
	
	function __getPT($srv,$refresh=false){
		
		$AskedPT = false;
		
		if(empty($_SESSION['cas']['cache_pt']) 
				|| $refresh) {
			$AskedPT = true;
			$pt = phpCAS::retrievePT($srv,$err_code,$output);
			$_SESSION['cas']['cache_pt'] = $pt;
		}
		$pt = $_SESSION['cas']['cache_pt'];
		
		if(empty($pt) || (!preg_match('/^PT-/',$pt) && !preg_match('/^ST-/',$pt)) )  {
			Horde::logMessage("bad Ticket was retrieve : $pt", __FILE__, __LINE__, LOG_ERROR);
			$this->__refreshAuth();
		}	
		
		if($AskedPT) {
			Horde::logMessage("User : ".phpCAS::getUser()." retrieve Ticket  : $pt", __FILE__, __LINE__, LOG_NOTICE);
		}
		return($pt);
	}
	
	function __refreshAuth(){
		
		unset($_SESSION['phpCAS']);
		unset($_SESSION['cas']);
		unset($_GET['ticket']);
		$this->__sav_arg();
		
		phpCAS::renewAuthentication();
	}
	
	function __getIMPVars(){
		if(empty($this->_imapService) || empty($this->_imapRealm)) {
			@define ('IMP_BASE', HORDE_BASE . '/imp');
			include IMP_BASE . '/config/servers.php';
			if (!empty($GLOBALS['conf']['vhosts'])) {
				include_once IMP_BASE . '/config/servers-' . $GLOBALS['conf']['server']['name'] . '.php';
			}
			require_once IMP_BASE . '/lib/IMP.php';
			
			$server = IMP::getAutoLoginServer();
			if(!empty($server)) {
				$p = preg_replace("'/(.*)$'" , "" , $servers[$server]['protocol']);		
				
				$this->_imapService = $p."://".$servers[$server]['server'];
				$this->_imapRealm = $servers[$server]['realm'];
				$this->_imapHordeAuth = $servers[$server]['hordeauth'];
			}
		}
	}
	
	function __sav_arg() {    	
		if(empty($_GET['pgtId'])){
			if(empty($_SESSION['cas']['__SAVARG']) 
					|| !is_array($_SESSION['cas']['__SAVARG']) ) {
				
				if(!empty($_GET) 
						&& is_array($_GET)) $SAV['_GET'] = $_GET;
				if(!empty($_POST) 
						&& is_array($_POST)) $SAV['_POST'] = $_POST;
				
				$_SESSION['cas']['__SAVARG_FILE'] = $this->__tempFilePath();
				
				$fp = @fopen($_SESSION['cas']['__SAVARG_FILE'], 'w');      
				if(!$fp) {
					Horde::logMessage("failed to open save argument file : ".$_SESSION['cas']['__SAVARG_FILE'], __FILE__, __LINE__, LOG_ERROR);
					unset($_SESSION['cas']['__SAVARG_FILE']);
					return(false);
				}
				fwrite( $fp, serialize($SAV));
				fclose($fp);
			}
		}
	}
	
	function __get_arg() {
		if(!empty($_SESSION['cas']['__SAVARG_FILE']) 
				&& file_exists($_SESSION['cas']['__SAVARG_FILE'])) {
			
			$fp = @fopen($_SESSION['cas']['__SAVARG_FILE'], 'r');
			if(!$fp) {
				Horde::logMessage("failed to open save argument file : ".$_SESSION['cas']['__SAVARG_FILE'], __FILE__, __LINE__, LOG_ERROR);
				unset($_SESSION['cas']['__SAVARG_FILE']);
				return(false);
			}
			$serial_sav = fread($fp, filesize ($_SESSION['cas']['__SAVARG_FILE']));
			
			$SAV = unserialize($serial_sav);
			@unlink($_SESSION['cas']['__SAVARG_FILE']);
			
			if(!empty($SAV['_GET']) 
					&& is_array($SAV['_GET'])) {
				
				reset ($SAV['_GET']);
				while (list ($key, $val) = each ($SAV['_GET'])) {
					if(empty($_GET[$key])){
						if($key == 'url') {
							$val = preg_replace("/&amp;/","&",$val);
							$val = Horde::applicationUrl($val,true,false);
							
						}
						$_GET[$key] = $val;
					}
				}
			}
			
			if(!empty($SAV['_POST']) 
					&& is_array($SAV['_POST'])) {
				
				reset ($SAV['_POST']);
				while (list ($key, $val) = each ($SAV['_POST'])) {
					if(empty($_POST[$key])) { 
						if($key == 'url') {
							$val = preg_replace("/&amp;/","&",$val);
							$val = Horde::applicationUrl($val,true,false);
							
						}
						$_POST[$key] = $val;
					}
				}
			}
			$_SESSION['cas']['__SAVARG'] = '';
			unset($_SESSION['cas']['__SAVARG']);
		}
	} 
	
	function __tempFilePath() {
		$tmpfile = Horde::getTempFile('cas_', false);
		return($tmpfile);
	}
	
	/**
	 * Returns the status string to use for logout messages.
	 *
	 * @access public
	 *
	 * @return string  The logout reason string.
	 */
	function getLogoutReasonString()
		{
		switch (Auth::getLogoutReason()) {
			case AUTH_REASON_BADPERM:
				$deconnect = phpCAS::getServerLogoutURL().'?service='.preg_replace('/&/','%26',phpCAS::getServiceURL());
				$text = sprintf(_("<a href='%s'>You don't have right to use this application. You must deconnect here</a>"),$deconnect); 
				break;
			default:
				$text = $auth = parent::getLogoutReasonString();     
			break;
		}
		
		return $text;
		}
}
